import gc
import os
import sys
import time

import connectionchecker
import soundstuff
import microcontroller
from adafruit_hid.keycode import Keycode
from adafruit_hid.consumer_control_code import ConsumerControlCode
from BrailleInput_Table import brailleinputtable
import appsounds
from FreeMemInfo import meminfostr

lowercasechars = "abcdefghijklmnopqrstuvwxyz"


class CommandFunctions:
    def __init__(self, globaldata):
        self.globaldata = globaldata
        self.keyboard = None
        self.pwd = None
        self.ble = None
        self.connectionchecker = None
        self.commandhandler = None
        self.braillekeyboard = None
        self.batt_avarage = None
        self.keylocked = False
        self.testmode = False
        self.testmacro = False
        self.volttime = 0
        self.commandstr = ""
        self.keycollector = None
        self.keycollection = []
        self.mousestate = 1
        self.mousex = 0
        self.mousey = 0
        self.soundstate = 0

    def prepare(self, hidkeyboard, pwd, ble, connectionchecker, batt_avarage):
        self.keyboard = hidkeyboard
        self.pwd = pwd
        self.ble = ble
        self.connectionchecker = connectionchecker
        self.batt_avarage = batt_avarage

    def backgroundcall(self):
        if self.volttime > 0:
            acttime = time.monotonic()
            if acttime - self.volttime > 10:
                self.volt()
                self.volttime = acttime

    def doeval(self, evalstr):
        gc.collect()
        eval(evalstr, {"self": self})

    def power_off(self):
        self.keyboard.press(Keycode.CONTROL)
        time.sleep(0.2)
        self.keyboard.press(Keycode.COMMAND)
        time.sleep(0.2)
        self.keyboard.press(Keycode.Q)
        time.sleep(0.2)
        self.releasekeys()
        time.sleep(0.2)

    def releasekeys(self):
        self.keyboard.release_all()

    def unlock(self):
        print("send code", self.pwd)
        self.keyboard.singlekey(Keycode.HOME)
        for c in self.pwd:
            if c == ",":
                time.sleep(0.15)
            elif c == "<":
                self.keyboard.singlekey(Keycode.BACKSPACE)
            else:
                self.keyboard.sendchar(c)

    def vo_fastnavtoggle(self):
        self.keyboard.keycombo((Keycode.LEFT_ARROW, Keycode.RIGHT_ARROW))

    def vo_doubletip(self):
        self.keyboard.keycombo((Keycode.UP_ARROW, Keycode.DOWN_ARROW))

    def vo_appswitcher(self):
        self.keyboard.key(Keycode.H, self.commandstr, voiceover=True, secondkey=Keycode.H)

    def enter(self):
        self.keyboard.singlekey(Keycode.ENTER)

# internal
    def _mousekey(self, command):
        self.keycollection.append(command)
        print("mousekey", command, len(self.keycollection))
        if command.isdigit():
            if self.mousestate == 1:
                self.mousey = self.mousey * 10 + int(command)
                print("mousey", self.mousey)
            elif self.mousestate == 2:
                self.mousex = self.mousex * 10 + int(command)
                print("mousex", self.mousex)
        elif command == ",":
            print("komma!", self.mousestate)
            if self.mousestate == 1:
                print("switch to 2")
                self.mousestate = 2
            print(self.mousestate)
        elif command == " ":
            # print("a mouse to:", -1000, -1000)
            # self.keyboard.mouse.move(-1000, -1000)
            # time.sleep(0.5)
            self.keyboard.mouse.move(self.mousex, self.mousex)
            print("done!")
            self.keycollector = None

    def mouse(self):
        self.keycollector = self._mousekey
        self.keycollection = []
        self.mousestate = 1
        self.mousex = 0
        self.mousey = 0

    def mouseagain(self):
        print("a mouse to:", -1000, -1000)
        self.keyboard.mouse.move(-1000, -1000)
        time.sleep(0.5)
        print("b mouse to:", self.mousey, self.mousex)
        self.keyboard.mouse.move(self.mousex, self.mousey)

    def keyclick(self):
        self.globaldata.keyclick = not self.globaldata.keyclick
        print("keyclick", self.globaldata.keyclick)

    def nosound(self):
        self.keycollector = None
        self.soundstate = 0

    def _playsound(self, command):
        ch = command.lower()
        if ch in lowercasechars:
            nr = ord(ch)-ord("a")+1
        elif ch in "123456789":
            nr = ord(ch)-ord("1")+31
        elif ch == "0":
            nr = 40
        else:
            return
        appsounds.playsoundnr(nr, self.globaldata)
        if self.soundstate == 1:
            self.nosound()
            return

    def playsound(self):
        if self.soundstate == 0:
            self.soundstate = 1
            self.keycollector = self._playsound
        elif self.soundstate == 1:
            self.soundstate = 2
        else:
            self.nosound()

    def _selectpartner(self, command):
        ch = command.lower()
        self.keycollector = None
        if ch in " abcd*":
            if ch == " ":
                nr = 0
            elif ch == "*":
                nr = 9
            else:
                nr = ord(ch)-ord("a")+1
            if nr < 0:
                nr = 0
            self.globaldata.setcurrentpartner(nr)
            print("")
            print(" "*20, nr, "selected")
            print("")
        else:
            print("error_selectpartner")

    def selectpartner(self):
        self.keycollector = None
        self.keycollector = self._selectpartner

    def _setcurrentasnr(self, command):
        self.keycollector = None
        ch = command.lower()
        if ch in "abcd":
            nr = ord(ch)-ord("a")+1
            print("")
            self.globaldata.setpartnerid(nr, self.connectionchecker.currentconnectionid)
            print("")
        else:
            print("error_setcurrentasnr")

    def setcurrentas(self):
        if len(self.connectionchecker.currentconnectionid) > 1:
            self.keycollector = self._setcurrentasnr

    def _setkbi(self, command):
        print("_setkbi", command)
        ch = command.lower()
        self.keycollector = None
        if ch in lowercasechars:
            self.globaldata.setkbi(ch)
            self.reload()

    def setkbi(self):
        self.keycollector = self._setkbi

    def _setkbc(self, command):
        print("_setkbc", command)
        ch = command.lower()
        self.keycollector = None
        if ch in lowercasechars:
            self.globaldata.setkbc(ch)
            self.reload()

    def setkbc(self):
        self.keycollector = self._setkbc

    def _setkbl(self, command):
        print("_setkbl", command)
        ch = command.lower()
        self.keycollector = None
        if ch in lowercasechars:
            self.globaldata.setkbl(ch)
            self.reload()

    def setkbl(self):
        self.keycollector = self._setkbl

    def keylayout(self):
        self.keyboard.singlekey(Keycode.ENTER)
        self.keyboard.sendstr(" " + self.keyboard.keyboardlayout + " ")

    def keylock(self):
        self.keylocked = not self.keylocked
        if self.keylocked:
            soundstuff.playsound(self.globaldata.lockedtone)
            print("keyboard now locked")
        else:
            soundstuff.playsound(self.globaldata.unlockedtone)
            print("keyboard now free")

    def version(self):
        print("send version", self.globaldata.deviceversion)
        self.keyboard.singlekey(Keycode.ENTER)
        self.keyboard.sendstr(self.globaldata.deviceversion)

    def volt(self):
        volt = f"{self.batt_avarage.avgvalue}"
        print("send volt", volt)
        self.keyboard.singlekey(Keycode.ENTER)
        self.keyboard.sendstr(volt)

    def batterystate(self):
        a = self.batt_avarage.avgvalue
        # a = 37000
        print("batterystate", a, self.batt_avarage.trend, self.batt_avarage.ref)
        if a < 35000:
            soundstuff.playsound(self.globaldata.battempty)
        elif a < 37000:
            soundstuff.playsound(self.globaldata.battlow)
        elif a < 39000:
            soundstuff.playsound(self.globaldata.batthalf)
        else:
            soundstuff.playsound(self.globaldata.battfull)
        if self.batt_avarage.trend < -3000:
            time.sleep(0.5)
            soundstuff.playsound(self.globaldata.battdischarge)
        if self.batt_avarage.trend > 3000:
            time.sleep(0.5)
            soundstuff.playsound(self.globaldata.battcharge)

    def voltstat(self):
        self.volttime = time.monotonic()

    def disconnect(self):
        self.connectionchecker.wantconnection = False

    def reconnect(self):
        self.connectionchecker.wantconnection = True

    def shutdown(self):
        print("shutdown")
        soundstuff.playsound(self.globaldata.resettone)
        print("disconnect")
        print("!!!!!!!!!!!!!!!!!! stop_advertising 20 ")
        self.ble.stop_advertising()
        for c in self.ble.connections:
            c.disconnect()
            time.sleep(1)
        time.sleep(0.5)

    def reset(self):
        self.shutdown()
        microcontroller.reset()

    def bootloader(self):
        self.shutdown()
        microcontroller.on_next_reset(microcontroller.RunMode.BOOTLOADER)
        microcontroller.reset()

    def reload(self):
        gc.collect()
        print(gc.mem_free())

        soundstuff.playsound(self.globaldata.reloadstarttone)
        if not self.keyboard.kl.load_layout_table(self.globaldata.getkbl()):
            soundstuff.playsound(self.globaldata.loadusernameerrortone)
        gc.collect()
        soundstuff.playsound(self.globaldata.reloadstarttone)
        if not brailleinputtable.load_input_table():
            soundstuff.playsound(self.globaldata.loadusernameerrortone)
        soundstuff.playsound(self.globaldata.reloadstarttone)
        gc.collect()
        if not brailleinputtable.load_binding_table():
            soundstuff.playsound(self.globaldata.loadusernameerrortone)
        gc.collect()

        soundstuff.playsound(self.globaldata.reloaddonetone)

    def showkeybinding(self):
        self.testmode = not self.testmode
        if self.testmode:
            soundstuff.playsound(self.globaldata.showstarttone)
        else:
            soundstuff.playsound(self.globaldata.showdonetone)

    def showmacro(self):
        self.testmacro = True
        soundstuff.playsound(self.globaldata.showstarttone)

    def loadmacros(self):
        print("loadmacros", meminfostr(True))
        rootdir = os.listdir()
        for f in rootdir:
            if ("macro_" in f) and (".txt" in f):
                ch = f[6:7]
                if ch in lowercasechars:
                    pos = ord(ch) - ord("a")
                    print(pos, f, meminfostr(True))
                    try:
                        with open(f, "r") as textfile:
                            lines = textfile.readlines()
                            i = 0
                            mlines = []
                            for line in lines:
                                line = line.strip()
                                ll = len(line)
                                if ll == 0:
                                    continue
                                elif ll == 1:
                                    i += 1
                                    print(i, line)
                                    mlines.append(line)
                                else:
                                    parts = line.splitt(" ")
                                    if len(parts) == 1:
                                        mlines.append(parts[0])
                                    else:
                                        mlines.append(parts[1])
                        self.globaldata.macros[pos] = mlines
                        mlines = 0
                        lines = 0
                        line = 0
                        print(meminfostr(True))
                    except Exception as e:
                        print("Error reading File:", str(e), f)
        self.globaldata.savemacros()
        print("done", meminfostr(True))

    def typehistory(self):
        return

    def tapandhold(self):
        self.keyboard.tapandhold()

    def computerbraille(self):
        # print("Set computerbraille")
        self.commandhandler.grade = 0
        soundstuff.playsound(self.globaldata.computerbraille)

    def vollschrift(self):
        # print("Set vollschrift")
        self.commandhandler.grade = 1
        soundstuff.playsound(self.globaldata.vollschriftbraille)

    def vollschrifttoggle(self):
        # print("Set vollschrifttoggle")
        if self.commandhandler.grade == 0:
            self.vollschrift()
        else:
            self.computerbraille()

    def setautorepeatthreshold1(self):
        self.commandhandler.clear_autorepeat_flags()
        self.commandhandler.nextsetautorepeat = True
        self.commandhandler.threshold1 = True

    def setautorepeatthreshold2(self):
        self.commandhandler.clear_autorepeat_flags()
        self.commandhandler.nextsetautorepeat = True
        self.commandhandler.threshold2 = True

    def setautorepeatkeydelay1(self):
        self.commandhandler.clear_autorepeat_flags()
        self.commandhandler.nextsetautorepeat = True
        self.commandhandler.keydelay1 = True

    def setautorepeatkeydelay2(self):
        self.commandhandler.clear_autorepeat_flags()
        self.commandhandler.nextsetautorepeat = True
        self.commandhandler.keydelay2 = True

    def repeatkeypress(self):
        self.commandhandler.nextrepeatkeypress = True
        self.commandhandler.repeatcount = 0

    def repeatagain(self):
        command = self.commandhandler.repeatcommand
        self.commandhandler.repeatcount = self.commandhandler.repeatlastc
        while self.commandhandler.repeatcount > 0:
            self.commandhandler.repeatcount -= 1
            self.commandhandler.docommand(command)
            print("sleep:", self.commandhandler.braillekeyboard.keydelay2)
            time.sleep(self.commandhandler.braillekeyboard.keydelay2)

    def macrorecord1(self):
        # print("do macrorecord1")
        self.commandhandler.macrorecording = True
        self.commandhandler.recordmode = 0
        self.commandhandler.commandlist = []
        soundstuff.playsound(self.globaldata.macrorecord1)

    def macrorecord2(self):
        # print("do macrorecord2")
        self.commandhandler.macrorecording = True
        self.commandhandler.recordmode = 1
        self.commandhandler.commandlist = []
        soundstuff.playsound(self.globaldata.macrorecord2)

    def macrostoprecord(self):
        # print("do macrostoprecord")
        print("CL:", self.commandhandler.commandlist)
        self.commandhandler.macrorecording = False
        self.commandhandler.recordmode = 0
        self.commandhandler.activemacro = []
        for c in self.commandhandler.commandlist:
            self.commandhandler.activemacro.append(c)
        self.commandhandler.commandlist = []
        soundstuff.playsound(self.globaldata.macrostoprecord)
        print("AM:", self.commandhandler.activemacro)

    def typemacrocmd(self, c):
        print("typemacrocmd", c)
        self.enter()
        time.sleep(0.1)
        self.keyboard.sendstr(c + " ")
        time.sleep(0.1)

    def typeallmacros(self):
        self.keyboard.sendstr("macrolist:")
        self.enter()
        mc = ord("a")
        print("-----------------", "start all")
        for m in self.globaldata.macros:
            time.sleep(self.braillekeyboard.keydelay2)
            self.enter()
            time.sleep(self.braillekeyboard.keydelay2)
            self.keyboard.sendstr("macro_" + chr(mc))
            time.sleep(self.braillekeyboard.keydelay2)
            self.enter()
            time.sleep(self.braillekeyboard.keydelay2)
            print("m", m)
            pos = 0
            while True:
                try:
                    c = m[pos]
                    print(pos, c)
                    self.keyboard.sendstr(c)
                    time.sleep(self.braillekeyboard.keydelay2)
                    self.enter()
                    time.sleep(self.braillekeyboard.keydelay2)
                    pos += 1
                except Exception as e:
                    print("End play: ", str(mc), str(e))
                    break
            mc += 1
            print("mc+=1", mc)
        print("-----------------", "done all")

    def domacroplayactive(self):
        pos = 0
        while True:
            try:
                c = self.commandhandler.activemacro[pos]
                pos += 1
                if self.testmacro:
                    self.typemacrocmd(c)
                else:
                    self.commandhandler.docommand(c)
                time.sleep(self.braillekeyboard.keydelay2)
            except Exception as e:
                print("End play: ", str(e))
                if self.testmacro:
                    time.sleep(0.05)
                    self.enter()
                    time.sleep(0.05)
                    self.keyboard.singlekey(Keycode.CONTROL)
                    time.sleep(0.1)
                    self.keyboard.singlekey(Keycode.UP_ARROW)
                break
        self.testmacro = False
        return

    def storemacro(self):
        soundstuff.playsound(self.globaldata.storemacro)
        self.commandhandler.storemacro = True

    def playmacro(self):
        soundstuff.playsound(self.globaldata.playmacro)
        self.commandhandler.playmacro = True

    def pause(self):
        time.sleep(0.1)

# ------------- consumer control functions
    def eject(self):
        self.keyboard.send_cc(ConsumerControlCode.EJECT)

    def record(self):
        self.keyboard.send_cc(ConsumerControlCode.RECORD)  # 0xB2

    def fast_forward(self):
        self.keyboard.send_cc(ConsumerControlCode.FAST_FORWARD)  # 0xB3

    def rewind(self):
        self.keyboard.send_cc(ConsumerControlCode.REWIND)  # 0xB4

    def scan_next_track(self):
        self.keyboard.send_cc(ConsumerControlCode.SCAN_NEXT_TRACK)  # 0xB5

    def scan_previous_track(self):
        self.keyboard.send_cc(ConsumerControlCode.SCAN_PREVIOUS_TRACK)  # 0xB6

    def stop(self):
        self.keyboard.send_cc(ConsumerControlCode.STOP)  # 0xB7

    def play_pause(self):
        self.keyboard.send_cc(ConsumerControlCode.PLAY_PAUSE)  # 0xCD

    def mute(self):
        self.keyboard.send_cc(ConsumerControlCode.MUTE)  # 0xE2

    def volume_decrement(self):
        self.keyboard.send_cc(ConsumerControlCode.VOLUME_DECREMENT)  # 0xEA

    def volume_increment(self):
        self.keyboard.send_cc(ConsumerControlCode.VOLUME_INCREMENT)  # 0xE9

    def brightness_decrement(self):
        self.keyboard.send_cc(ConsumerControlCode.BRIGHTNESS_DECREMENT)  # 0x70

    def brightness_increment(self):
        self.keyboard.send_cc(ConsumerControlCode.BRIGHTNESS_INCREMENT)  # 0x6F

    def cchome(self):
        self.keyboard.send_cc(0x223)
