import gc
import sys
import time
import soundstuff
from adafruit_hid.keycode import Keycode
from commandfunctions import CommandFunctions


# char2digit = {"a":"1","b":"2","c":"3","d":"4","e":"5","f":"6","g":"7","h":"8","i":"9","j":"0",}

class CommandHandler:
    def __init__(self, hidkeyboard, commandfunctions, braillekeyboard, globaldata):
        self.globaldata = globaldata
        self.keyboard = hidkeyboard
        self.commandfunctions = commandfunctions
        self.braillekeyboard = braillekeyboard
        self.commandstr = ""
        self.commandhistory = []
        self.numvalue = 1
        # ----------------------------
        self.grade = 0
        self.allcap = False
        self.nextcap = False
        self.nextspecial = False
        self.numbermode = False
        self.numbersgenerated = False
        # ----------------------------
        self.nextsetautorepeat = False
        self.threshold1 = False
        self.threshold2 = False
        self.keydelay1 = False
        self.keydelay2 = False
        # ----------------------------
        self.nextrepeatkeypress = False
        self.repeatcount = 0
        self.repeatlastc = 0
        self.repeatcommand = ""
        # ----------------------------
        self.macrorecording = False
        self.recordmode = 0
        self.commandlist = []
        self.activemacro = ["a", "b", "c"]
        self.storemacro = False
        self.playmacro = False

    #def append2history(self, keystring, command):
        pass
        #while len(self.commandhistory) >= 100:
        #    self.commandhistory.pop(0)
        #gc.collect()
        #self.commandhistory.append((keystring, command))

    def grade1_flagsoff(self):
        self.allcap = False
        self.nextcap = False
        self.nextspecial = False
        self.numbermode = False

    def backgroundcall(self):
        pass

    def getnumvalue(self, command, playbad=True):
        scommand = command.lower()
                       #12345678901234567890123456
        if scommand in "abcdefghijklmnopqrstuvwxyz":
            self.numvalue = ord(scommand) - ord("a") + 1
            print("numvalue", self.numvalue)
            return True
        else:
            if playbad:
                soundstuff.playsound(self.globaldata.userpassworderrortone)
            return False

    def clear_autorepeat_flags(self):
        self.nextsetautorepeat = False
        self.threshold1 = False
        self.threshold2 = False
        self.keydelay1 = False
        self.keydelay2 = False

    def execute(self, keystr, command):
        # self.append2history(keystr, command)
        evalstr = "~"
        # print("execute", keystr, "command:", command, "len(command)", len(command))
        self.commandfunctions.volttime = 0
        if self.commandfunctions.testmode:
            self.commandfunctions.enter()
            if len(command) == 0:
                self.keyboard.sendstr(keystr + " = ")
            elif len(command) == 1:
                self.keyboard.sendstr(keystr + " = " + str(ord(command)) + " ")
            else:
                funcname = command.strip("<")
                funcname = funcname.strip(">")
                self.keyboard.sendstr(keystr + " = " + funcname + " ")
                if command == "<showkeybinding>":
                    self.commandfunctions.showkeybinding()
            self.keyboard.singlekey(Keycode.CONTROL)
            time.sleep(0.3)
            self.keyboard.singlekey(Keycode.UP_ARROW)
            self.keyboard.singlekey(Keycode.DOWN_ARROW)
            return
        if len(command) == 0:
            return
        if self.nextsetautorepeat:
            print("-- nextsetautorepeat", self.threshold1, self.threshold2, self.keydelay1, self.keydelay2)
            if len(command) == 1:
                print("       command:"+command+"~")
                if self.threshold1:
                    if self.getnumvalue(command):
                        self.braillekeyboard.threshold1 = self.numvalue
                    print("threshold1:", self.braillekeyboard.threshold1)
                elif self.threshold2:
                    if self.getnumvalue(command):
                        self.braillekeyboard.threshold2 = self.numvalue
                    print("threshold2:", self.braillekeyboard.threshold2)
                elif self.keydelay1:
                    if self.getnumvalue(command):
                        self.braillekeyboard.keydelay1 = 1 / self.numvalue
                    print("keydelay1:", self.braillekeyboard.keydelay1)
                elif self.keydelay2:
                    if self.getnumvalue(command):
                        self.braillekeyboard.keydelay2 = 1 / self.numvalue
                    print("keydelay2:", self.braillekeyboard.keydelay2)
                else:
                    print("!!!!!!!!!! Nothing selected ??????")
                soundstuff.playsound(self.globaldata.keyclicktone)
            else:
                soundstuff.playsound(self.globaldata.userpassworderrortone)
            self.clear_autorepeat_flags()
            return
        if self.nextrepeatkeypress:
            print("------------------------------------ nextrepeatkeypress", self.repeatcount)
            print("       command:"+command+"~")
            if self.repeatcount == 0:
                if len(command) == 1:
                    if self.getnumvalue(command):
                        self.repeatcount = self.numvalue
                        soundstuff.playsound(self.globaldata.keyclicktone)
                    else:
                        soundstuff.playsound(self.globaldata.userpassworderrortone)
                        self.nextrepeatkeypress = False
                        self.repeatcount = 0
                    print("repeatcount:", self.repeatcount)
                else:
                    soundstuff.playsound(self.globaldata.userpassworderrortone)
                    self.nextrepeatkeypress = False
                    self.repeatcount = 0
            else:
                self.nextrepeatkeypress = False
                print("Do the repeat !!!", self.repeatcount)
                self.repeatlastc = self.repeatcount
                self.repeatcommand = command
                while self.repeatcount > 0:
                    self.repeatcount -= 1
                    self.docommand(command)
                    print("sleep:", self.braillekeyboard.keydelay2)
                    time.sleep(self.braillekeyboard.keydelay2)
            return
        if self.storemacro:
            self.storemacro = False
            print("------------------------------------ storemacro")
            print("       store:"+command+"~")
            if len(command) == 1:
                if self.getnumvalue(command, False):
                    print("store to:", self.numvalue, self.activemacro)
                    self.globaldata.macros[self.numvalue-1] = self.activemacro
                    self.globaldata.savemacros()
                    soundstuff.playsound(self.globaldata.storemacrodone)
                else:
                    soundstuff.playsound(self.globaldata.macrocancelrecord)
            else:
                soundstuff.playsound(self.globaldata.macrocancelrecord)
            return
        if self.playmacro:
            self.playmacro = False
            print("------------------------------------ playmacro")
            print("       play:"+command+"~")
            if len(command) == 1:
                if self.getnumvalue(command, False):
                    print("play from:", self.numvalue)
                    self.activemacro = self.globaldata.macros[self.numvalue-1]
                    soundstuff.playsound(self.globaldata.playmacrostart)
                    self.commandfunctions.domacroplayactive()
                else:
                    soundstuff.playsound(self.globaldata.macrocancelrecord)
            else:
                soundstuff.playsound(self.globaldata.macrocancelrecord)
            return
        if self.macrorecording:
            print("macrorecording", command)
            if command == "<storemacro>":
                self.commandfunctions.macrostoprecord()
                self.commandfunctions.storemacro()
            if command == "<macrostoprecord>":
                self.docommand(command)
                return
            if command == "<macrorecord1>":
                return
            if command == "<macrorecord2>":
                return
            self.commandlist.append(command)
            print("commandlist", self.commandlist)
            if self.recordmode == 0:  # no action
                return
        self.docommand(command)

    def docommand(self, command):
        pp = 1
        if len(command) == 1:
            if self.commandfunctions.keylocked:
                soundstuff.playsound(self.globaldata.lockedtone)
                return True
            else:
                if self.commandfunctions.keycollector:
                    print("collected !")
                    self.commandfunctions.keycollector(command)
                else:
                    if self.grade == 0:
                        pass
                    elif self.grade == 1:    # --- Ersetzungen
                        if command == "8":
                            command = "ü"
                        elif command == "`":
                            command = "ä"
                        elif command == "9":
                            command = "ö"
                        elif command == "~":
                            command = "ß"
                        elif command == "!":
                            command = "_"
                        elif command == "+":
                            command = "!"
                        elif command == "}":   # ----Kürzungen
                            command = "st"
                        elif command == "1":
                            command = "au"
                        elif command == "2":
                            command = "eu"
                        elif command == "|":
                            command = "äu"
                        elif command == "3":
                            command = "ei"
                        elif command == "4":
                            command = "ch"
                        elif command == "5":
                            command = "sch"
                        elif command == "0":
                            command = "ie"
                        elif command == "'":    # ---- Aufhebungszeichen.
                            command = ""        # B6 '
                            self.grade1_flagsoff()
                        elif command == "$":    # ---- Ankündigungszeichen.
                            command = ""        # B46 $ next cap
                            self.grade1_flagsoff()
                            self.nextcap = True
                        elif command == ">":
                            command = ""        # B45 > all caps
                            self.grade1_flagsoff()
                            self.allcap = True
                        elif command == '"':
                            command = ""        # B4 "  next special
                            self.grade1_flagsoff()
                            self.nextspecial = True
                        elif command == "#":
                            if self.numbermode:
                                print("set Numbermode = False")
                                self.numbermode = False
                                if self.numbersgenerated:
                                    command = ""
                            else:
                                print("set Numbermode = True")
                                self.numbermode = True
                                self.numbersgenerated = False
                        # --- different modes are handeled here
                        if self.numbermode:
                            if command == "#":
                                command = ""
                            elif command == "a":
                                command = "1"
                                self.numbersgenerated = True
                            elif command == "b":
                                command = "2"
                                self.numbersgenerated = True
                            elif command == "c":
                                command = "3"
                                self.numbersgenerated = True
                            elif command == "d":
                                command = "4"
                                self.numbersgenerated = True
                            elif command == "e":
                                command = "5"
                                self.numbersgenerated = True
                            elif command == "f":
                                command = "6"
                                self.numbersgenerated = True
                            elif command == "g":
                                command = "7"
                                self.numbersgenerated = True
                            elif command == "h":
                                command = "8"
                                self.numbersgenerated = True
                            elif command == "i":
                                command = "9"
                                self.numbersgenerated = True
                            elif command == "j":
                                command = "0"
                                self.numbersgenerated = True
                            elif command == ",":
                                command = ","
                                self.numbersgenerated = True
                            elif command == " ":
                                self.numbermode = False
                                if not self.numbersgenerated:
                                    command = "#"
                            else:
                                self.numbermode = False
                        print(self.numbermode, self.numbersgenerated)
                    while len(command) > 0:
                        ch = command[0]
                        command = command[1:]
                        if self.nextcap:
                            self.nextcap = False
                            ch = ch.upper()
                        elif self.allcap:
                            ch = ch.upper()
                        elif self.nextspecial:
                            self.nextspecial = False
                            if ch == "`":
                                ch = "@"
                            elif ch == "e":
                                ch = "€"
                            elif ch == "|":
                                ch = "\\"
                            elif ch == ",":
                                ch = "/"
                        self.keyboard.sendchar(ch)
                        if len(command) > 0:
                            self.braillekeyboard.checkkeys()
                            time.sleep(self.braillekeyboard.keydelay2)
                return True
        # print("execute:", time.monotonic(), command)
        self.commandstr = command
        funcname = command.strip("<")
        funcname = funcname.strip(">")
        if funcname[0:4] == "str:":
            self.keyboard.sendstr(funcname[4:])
        if funcname[0:4] == "key_":
            #                01234
            funcparam = funcname[4:]
            # print("funcparam", funcparam)
            funclist = funcparam.split("-")
            # print("funclist", funclist)
            code = 0
            control = False
            option = False
            shift = False
            cmd = False
            insert = False
            capslock = False
            voiceover = False
            jaws = False
            nvda = False
            for i, s in enumerate(funclist):
                if i == 0:
                    evalstr = "Keycode." + s.upper()
                    # print("evalstr:", evalstr)
                    try:
                        code = eval(evalstr)
                    except Exception as e:
                        print("error:", str(e), evalstr)
                        pass
                else:
                    if s == "o":
                        option = True
                    elif s == "^":
                        control = True
                    elif s == "s":
                        shift = True
                    elif s == "c":
                        cmd = True
                    elif s == "i":
                        insert = True
                    elif s == "l":
                        capslock = True
                    elif s == "v":
                        voiceover = True
                    elif s == "j":
                        jaws = True
                    elif s == "n":
                        nvda = True
            if code > 0:
                # print("key:", code, option, control, shift, cmd, insert, capslock, voiceover, jaws, nvda)
                self.keyboard.key(code, command, optionkey=option, controlkey=control, shiftkey=shift, commandkey=cmd, capslock=capslock, insert= insert, voiceover=voiceover, jaws=jaws, nvda=nvda)
            # print("result:", code, shift, cmd)
        else:
            pp = 13
            # try:
            try:
                evalstr = "self." + funcname + "()"
                print(evalstr, "keylocked:", self.commandfunctions.keylocked)
                pp = 14
                if self.commandfunctions.keylocked:
                    if funcname != "keylock":
                        soundstuff.playsound(self.globaldata.lockedtone)
                        return True
                pp = 15
                self.commandfunctions.doeval(evalstr)
                # eval(evalstr, {"self": self})
                return True
            except Exception as e:
                print("Error execute: ", str(e), pp, evalstr)
                return False

