import time
from adafruit_ble.services.standard.device_info import DeviceInfoService
import soundstuff


class ConnectionChecker:
    def __init__(self, service, ble, advertisement, globaldata):
        self.service = service
        self.ble = ble
        self.advertisement = advertisement
        self.currentconnectionid = ""
        self.isconnected = False
        self.wasconnected = False
        self.wantconnection = True
        self.wasadvertising = False
        self.needbeep = False
        self.start_advertisingtime = 0
        self.globaldata = globaldata

    def start_advertising(self, nr):
        if not self.ble.advertising:
            print("!!!!!!!!!!!!!!!!!!! Start advertizing:", nr)
            self.wasadvertising = True
            self.ble.start_advertising(self.advertisement)
            self.start_advertisingtime = time.monotonic()

    def stop_advertising(self, nr):
        if self.wasadvertising:
            self.wasadvertising = False
            print("!!!!!!!!!!!!!!!!!! stop_advertising", nr)
            self.ble.stop_advertising()

    def connectionid(self, c):
        try:
            if DeviceInfoService in c:
                try:
                    dis = c[DeviceInfoService]
                    s = ""
                    try:
                        s += dis.manufacturer
                    except:
                        s += "-man"
                    try:
                        s += dis.model_number
                    except:
                        s += "-mod"
                    #try:
                    #    s += dis.serial_number
                    #except:
                    #    s += "-#"
                except Exception as e:
                    s = "-err"
            else:
                s = "?"
        except Exception as e:
            s = "!"
        return s

    def onlyoneconnection(self):
        #print("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv")
        self.isconnected = False
        pname = self.globaldata.getpartnerid()
        if not self.ble.connected:
            # print(f"not self.ble.connected. advertising:{self.ble.advertising}  looking for: {pname}")
            # print("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~")
            self.currentconnectionid = "!"
            if not self.ble.advertising:
                soundstuff.playsound(self.globaldata.disconnectedtone)
                self.needbeep = True
            # print("start_advertising")
            self.start_advertising(101)
            return
        connections = self.ble.connections
        cns = "|"
        for c in connections:
            if pname == "*":
                c.cname = pname
            else:
                try:
                    s = c.cname
                    if s == "*":
                        c.cname = self.connectionid(c)
                except:
                    c.cname = self.connectionid(c)
            cns += c.cname + "|"
        print(cns)
        if (pname == "-all") or (pname == "*") :
            if len(connections) == 0:  # not sure if possible !
                #print("no connections -> start advertising")
                self.start_advertising(102)
                self.needbeep = True
            self.start_advertisingtime = 0  # signal no timeout
            elapsedtime = -1
        else:
            needadv = True
            for c in connections:
                if pname == c.cname:
                    needadv = False  # desired device already in the list
            if needadv:
                self.start_advertising(102)
                self.needbeep = True  # not in the current list beep later
            else:
                self.start_advertisingtime = 0
            if self.start_advertisingtime != 0:
                self.start_advertising(102)
        if self.start_advertisingtime > 0:
            elapsedtime = abs(time.monotonic() - self.start_advertisingtime)
            #print(f"looking for {pname} since {elapsedtime} seconds", self.currentconnectionid)
        else:
            elapsedtime = -1
            #print(f"connected to {self.currentconnectionid} wanted {pname}")
        fname = ""
        for c in connections:
            cname = c.cname
            if pname == "-all":
                fname = cname
                break
            else:
                if pname == cname:
                    fname = cname
                    break
        if fname:
            if self.ble.advertising:
                self.stop_advertising(99)
                time.sleep(1)
            #print(f"found partner {fname} after {elapsedtime} seconds")
            #print(f"now {len(connections)} connections")
            for c in connections:
                if c.cname == fname:
                    if len(connections) == 1:
                        self.isconnected = True
                        # print("set self.isconnected = True")
                        if self.needbeep:
                            self.needbeep = False
                            self.currentconnectionid = fname
                            print("conn", cname, "!!!!!!!!!!!!!!!!!!!!!")
                            soundstuff.playsound(self.globaldata.connectedtone)
                elif c.cname:
                    print("disc", c.cname, ">>>>>>>>>>>>>>>>>>>>")
                    c.disconnect()
                else:
                    print("disc", c.cname, "????????????????????")
        else:
            if elapsedtime > 10:
                print("timeout !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!")
                self.globaldata.setcurrentpartner(9)
                self.currentconnectionid = "?"

    def check(self):
        if self.wantconnection:
            self.onlyoneconnection()
        else:  # no connections wanted !
            print("no connections wanted!")
            if self.isconnected:
                print("if still connected")
                # if still connected & advertising
                if self.ble.advertising:
                    print("if still advertising")
                    self.stop_advertising(12)
                    time.sleep(1)
                dosound = False
                for c in self.ble.connections:
                    time.sleep(0.5)
                    c.disconnect()
                    dosound = True
                if dosound:
                    print("at least one disconnected")
                    soundstuff.playsound(self.globaldata.disconnectedtone)
                    self.isconnected = False
                else:
                    print("nothing disconnected")
            self.wasconnected = False
