# SPDX-FileCopyrightText: 2017 Dan Halbert for Adafruit Industries
#
# SPDX-License-Identifier: MIT

"""
`adafruit_hid.keyboard_layout_de.KeyboardLayoutDE`
=======================================================

* Author(s): Dan Halbert & Thomas Friehoff Verbesserung KeycodStrings durch Nummern ersetzen
"""

import os
import time
import json
from .keycode import Keycode


class KeyboardLayout:
    SHIFT_FLAG = 0x80
    CONTROL_FLAG = 0x100
    ALT_FLAG = 0x200

    def __init__(self, keyboard):
        self.keyboard = keyboard
        self.chr2keys = None
        self.isIOS = False

    def load_layout(self, directory):
        if not self.load_layout_table(directory):
            if not self.load_layout_table("\\"):
                return False
        return True

    def load_layout_table(self, directory):
        longprint = False
        if len(directory) <= 1:
            path = directory
        else:
            path = directory[0:-1]
        print("listdir", path)
        dirlist = os.listdir(path)
        fn = ""
        for filename in dirlist:
            print(filename)
            if ("keyboard_" in filename.lower()) and (".dat" in filename.lower()):
                fn = filename
                break
        if fn == "":
            print("not found!")
            return False
        if "ios" in fn.lower():
            self.isIOS = True
        else:
            self.isIOS = False
        filename = directory + fn
        print("load_layout", filename)
        try:
            with open(filename, "r") as textfile:
                try:
                    self.chr2keys = textfile.readline()
                except Exception as e:
                    print("Error reading file", str(e), )
        except Exception as e:
            print("Error opening File:", str(e), filename)
            return False
        print("load_layout done.")
        return True

    def write(self, string):
        print(" write:", string, '~',  end="")
        c = 0
        for char in string:
            if c > 0:
                time.sleep(0.1)
            c += 1
            # print("char:", char)
            keycode = self._char_to_keycode(char)
            print(type(keycode), end="")
            if keycode is None:
                continue
            elif type(keycode) == int:
                print("I", keycode, hex(keycode), end="")
                # If this is a shifted char, clear the SHIFT flag and press the SHIFT key.
                if keycode & self.SHIFT_FLAG:
                    keycode &= ~self.SHIFT_FLAG
                    self.keyboard.press(Keycode.SHIFT)
                self.keyboard.press(keycode)
            elif type(keycode) == tuple:
                for key in keycode:
                    print("T", key, hex(key), end="")
                    if key == 0:
                        self.keyboard.release_all()
                    else:
                        self.keyboard.press(key)
            elif type(keycode) == list:
                print("do list", keycode, end="")
                self.keyboard.press(Keycode.ALT)
                self.keyboard.press(Keycode.ALT)
                time.sleep(0.1)
                for key in keycode:
                    print("L", key, hex(key), end="")
                    self.keyboard.press(key)
                    time.sleep(0.02)
                    self.keyboard.release(key)
                    time.sleep(0.02)
            self.keyboard.release_all()

    def keycombo(self, keytuple):
        for key in keytuple:
            self.keyboard.press(key)
        self.keyboard.release_all()

    def _char_to_keycode(self, char):
        if len(char) > 1:
            print( "????", char, end="")
            return 0
        print(" ch2keyc.:", char, ord(char), hex(ord(char)), end="")
        st = "#"+str(ord(char))+":"
        fp = self.chr2keys.find(st)
        if fp >= 0:
            sp = self.chr2keys.find(":", fp + 1) + 1
            ep = self.chr2keys.find("#", fp + 1)
            # print(" ", sp, ep, end="")
            outstr = self.chr2keys[sp:ep]
            print(" " + outstr, end="")
            if outstr.isdigit():
                keycode = int(outstr)
                if keycode == 0:
                    raise ValueError("No keycode available for character.")
                return keycode
            else:
                keycode = eval(outstr)
                print(type(keycode), keycode, end="")
                for key in keycode:
                    print(" key:", key, hex(key), end="")
                return keycode
