import time
import sys
import pwmio
import board
from digitalio import DigitalInOut, Direction, Pull
import audiocore
from audiopwmio import PWMAudioOut as AudioOut

pin5 = None

class Sequencer:

    def __init__(self):
        self.tonelen = 0.001

    def playone(self, s):
        # <von>-<bis>:<speed>
        # print("playone:", s)
        try:
            count = 1
            print(s)
            s = s.split(":")
            if len(s) == 2:
                speed = int(s[1]) / 1000
            else:
                # print("default speed")
                speed = self.tonelen
            # print("split -", s)
            s = s[0].split("-")
            if len(s[0]) > 0:
                start = int(s[0])
            else:
                start = 100
            if len(s) == 2:
                finish = int(s[1])
            else:
                finish = start + 1
            if start < finish:
                d = 1
            else:
                d = -1
            print("pwm:", start, finish, d, speed)
            with pwmio.PWMOut(board.D5, duty_cycle=0x8000, frequency=start, variable_frequency=True) as pwm:
                for f in range(start, finish, d):
                    # print(f)
                    pwm.frequency = f
                    time.sleep(speed)
            print("pwm done")
        except Exception as ex:
            print("pwm exception:", str(ex))

    def playonemulti(self, s):
        try:
            s = s.split("#")
            count = 1
            if len(s) == 2:
                count = int(s[1])
            for i in range(0, count):
                self.playone(s[0])
        except Exception as ex:
            print("playonemulti:", str(ex))

    def playgroup(self, s):
        s = s.split("|")
        for o in s:
            self.playonemulti(o)

    def playgroupmulti(self, s):
        try:
            s = s.split("$")
            count = 1
            if len(s) == 2:
                count = int(s[1])
            for i in range(0, count):
                self.playgroup(s[0])
        except Exception as ex:
            print("playgroupmulti:", str(ex))

    def playlist(self, s):
        try:
            s = s.split(";")
            for i in s:
                self.playgroupmulti(i)
        except Exception as ex:
            print("playlist:", str(ex))


def tone(pin, frequency, duration=1):
    print("tone", pin, frequency, duration)
    try:
        with pwmio.PWMOut(pin, frequency=int(frequency), variable_frequency=False) as pwm:
            pwm.duty_cycle = 0x8000
            time.sleep(duration)
    except ValueError as e:
        print("no!  pwm", frequency, duration, str(e))


def pin5_quiet():
    global pin5
    pin5_free()
    try:
        pin5 = DigitalInOut(board.D5)
        pin5.switch_to_input(pull=Pull.DOWN)
    except  Exception as ex:
        print("pin5_free:", str(ex))


def pin5_free():
    global pin5
    if pin5 is None:
        return
    try:
        pin5.deinit()
    except  Exception as ex:
        print("pin5_free:", str(ex))


def playsound(s):
    pin2 = DigitalInOut(board.D2)
    pin2.switch_to_output()
    pin2.value = True
    print("playsound", s)
    pin5_free()
    if ("-" in s) or ("|" in s) or (";" in s) or (":" in s):
        seq.playlist(s)
    else:
        try:
            for f in s:
                # For the M4 boards:
                ss = "M4 " + str(f)
                print(ss, end=" ")
                if type(f) is tuple:
                    t = f[1]
                    f = f[0]
                    try:
                        d = f[2]
                    except:
                        d = 0.01
                else:
                    t = 0.08
                    d = 0.01
                print(t, f)
                if f > 0:
                    tone(board.D5, f, t)  # on for 1/4 second
                    time.sleep(d)  # pause between notes
                else:
                    time.sleep(d)
        except Exception as e:
            try:
                print("Err:", str(e), "other ", s)
                tone(board.D5, s, 0.08)  # on for 1/4 second
            except:
                print("Err in err :", str(e), "other ", s)
                pass
    pin5_quiet()
    pin2.value = False
    pin2.deinit()



def beep(freq, duration):
    pin5_free()
    tone(board.D5, freq, duration)
    pin5_quiet()


pin5_quiet()
seq = Sequencer()
